<?
/*

BSD license:

Copyright (c) 2009, KimikoMuffin
All rights reserved.

Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright notice, this
list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation and/or
other materials provided with the distribution.
 * Neither the name of "Muffiny Micellany" nor the names of its contributors
may be used to endorse or promote products derived from this software without
specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE DEVELOPER "AS IS" AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
SHALL THE DEVELOPER BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.

*/

error_reporting (E_COMPILE_ERROR|E_RECOVERABLE_ERROR|E_ERROR|E_CORE_ERROR); //Set error reporting

///////////////
// Constants //
///////////////

//Version.
define('VERSION','1.0.2');
define('DATE','20090930');
//Define max file size
define('MAXSIZE',0x100000);

//Header and footer
define('HEAD',<<<EOF
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
<head>
<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
EOF
.'<title>StegMuffin PHP'.VERSION.'/'.DATE.'</title>
</head>
<body>
<h1>StegMuffin PHP</h1>');
define('FOOT',"</body>\n</html>");


///////////////
// Functions //
///////////////

/*
//imgerr - displays a fatal error message. Used with 'or die(stegmuffin_imgerr(...))' construct.
   $what (string) - The nature of the error message. Complete the following: "Could not _____!"
*/
function stegmuffin_imgerr($what) {
  //RETURN: an error message including header and footer.
  die(HEAD."\n<h2>Error</h2>\n<p>Could not $what!</p>\n".FOOT);
}

/*
//showimg - Takes an image and gets the hidden image out of it.
   $oldImage (image) - the image with the hidden image.
   $bits (int) - The number of bits to use.
   $dload (bool) - If true, it will signal the browser to download the image. If not, the image will simply be displayed.
*/
function stegmuffin_showimg($oldImage, $bits, $dload = FALSE) {
  //Get the image's size.
  $imgsize = array(imagesx($oldImage),imagesy($oldImage)) or stegmuffin_imgerr('get image size');
  
  
  
  //Get a new image with the size of the old one!
  $newImage = imagecreatetruecolor($imgsize[0],$imgsize[1]) or stegmuffin_imgerr('create new image');
  
  //Deal with alpha.
  imagealphablending($newImage, false);
  imagesavealpha($newImage, true);
  
  //Set appropriate value for number of bits.
  $bittage = stegmuffin_bittify((int)$bits);
  
  //We don't like zeroes here, so ...
  if(!$bittage) stegmuffin_imgerr('properly set value for number of bits');
  
  //Two loops: one for each row, and one for each colum. Yes, that's right -- we're going to be altering the color of each individual pixel.
  for($y = 0; $y < $imgsize[1]; $y++) {
    for($x = 0; $x < $imgsize[0]; $x++) {
      //Get the old color.
      $oldColor = imagecolorat($oldImage,$x,$y);
      if($oldColor === FALSE) stegmuffin_imgerr('get old color at ('.$x.','.$y.')');
      
      //Get the least significant bits.
      $mColor = array(
	    'a' => ($oldColor & ($bittage * 0x1000000)) >> 24, //Alpha
	    'r' => ($oldColor & ($bittage * 0x10000)) >> 16, //Red
	    'g' => ($oldColor & ($bittage * 0x100)) >> 8, //Green
	    'b' => $oldColor & $bittage); //Blue
      unset($oldColor);
      
      //Create the amount to multiply the values by.
      $cmultiplier = (255 / $bittage);
      $amultiplier = (127 / $bittage);
      
      //Get the new color by multiplying it!
      $newColor = imagecolorallocatealpha($newImage,
		round(($mColor['r'] * $cmultiplier),0),	//Red
		round(($mColor['g'] * $cmultiplier),0),	//Green
		round(($mColor['b'] * $cmultiplier),0),	//Blue
		round(($mColor['a'] * $amultiplier),0)	//Alpha
		);
      //Error handling.
      if($newColor === FALSE) stegmuffin_imgerr('create new color at ('.$x.','.$y.'); values were R='.round($mColor['r'] * $cmultiplier,0).', G='.round($mColor['g'] * $cmultiplier,0).', B='.round($mColor['b'] * $cmultiplier,0).', A='.round(($mColor['a'] * $amultiplier),0));
      unset($cmultiplier);
      unset($amultiplier);
      unset($mColor);
      
      //Draws a single dot at the specified location!
      imagesetpixel($newImage,$x,$y,$newColor) or stegmuffin_imgerr('add new color at ('.$x.','.$y.')');
      //imagecolordeallocate: see under "unset" above.
      imagecolordeallocate($newImage,$newColor);
    }
  }
  //imagedestroy: see under "unset" above.
  imagedestroy($oldImage);
  //It's a PNG file.
  header('Content-type: image/png');
  //It wants you to download it.
  if($dload) header("Content-Disposition: attachment; filename=stegmuffin-show.png");
  //Shows the image.
  imagepng($newImage);
  //End of line.
  die;
}
/*
//hideimg - Takes two images and hides one in the other.
   $hideimg (image) - the image to be hidden.
   $coverimg (image) - the image to be hidden in.
   $bits (int) - The number of bits to use.
   $dload (bool) - If true, it will signal the browser to download the image. If not, the image will simply be displayed.
*/
function stegmuffin_hideimg($hidImage, $covImage, $bits, $dload = FALSE) {
  //Get the image's size.
  $imgsize = array(imagesx($covImage),imagesy($covImage)) or stegmuffin_imgerr('get image size');
  
  //Create new image.
  $newImage = imagecreatetruecolor($imgsize[0],$imgsize[1]) or stegmuffin_imgerr('create new image');
  
  //Deal with alpha.
  imagealphablending($newImage, false);
  imagesavealpha($newImage, true);
  
  //Set appropriate value for number of bits.
  $bittage = stegmuffin_bittify((int)$bits);
  
  //If it doesn't evaluate true, we don't want to do ANYTHING there, because the first thing we'd be doing is dividing by zero (OH SHI--)
  if(!$bittage) stegmuffin_imgerr('properly set value for number of bits');
  
  //Two loops: one for rows, one for columns.
  for($y = 0; $y < $imgsize[1]; $y++) {
    for($x = 0; $x < $imgsize[0]; $x++) {
      //Get old color.
      $hidColor = imagecolorat($hidImage,$x,$y);
      if($hidColor === FALSE) stegmuffin_imgerr('get color to hide at ('.$x.','.$y.')');
      
      //Create the amount to divide the values by.
      $cmultiplier = (255 / $bittage);
      $amultiplier = (127 / $bittage);
      
      //Split apart the hiding-image and darken it so it fits in the bittage.
      $hColor = array(
  	    'a' => round(((($hidColor & 0xFF00000) >> 24) / $amultiplier),0),
  	    'r' => round(((($hidColor & 0xFF0000) >> 16) / $cmultiplier),0),
	    'g' => round(((($hidColor & 0xFF00) >> 8) / $cmultiplier),0),
	    'b' => round((($hidColor & 0xFF) / $cmultiplier),0));
      unset($hidColor);
      unset($cmultiplier);
      unset($amultiplier);
      
      //Get the color of the corresponding point on the cover-image.
      $covColor = imagecolorat($covImage,$x,$y);
      if($covColor=== FALSE) stegmuffin_imgerr('get cover color at ('.$x.','.$y.')');
      
      //Set a value for what bits, of the new color, we do want.
      $divider = 0xFF - $bittage;
      
      //Shave off the unwanted bittage.
      $mColor = array(
	    'a' => (($covColor & ($divider * 0x1000000)) >> 24) + $hColor['a'],
	    'r' => (($covColor & ($divider * 0x10000)) >> 16) + $hColor['r'],
	    'g' => (($covColor & ($divider * 0x100)) >> 8) + $hColor['g'],
	    'b' => ($covColor & $divider) + $hColor['b']);
      unset($covColor);
      unset($divider);
      
      //Get the new color.
      $newColor = imagecolorallocatealpha($newImage,
			$mColor['r'],
			$mColor['g'],
			$mColor['b'],
			$mColor['a']);
      if($newColor === FALSE) stegmuffin_imgerr('create new color at ('.$x.','.$y.')');
      unset($mColor);
      
      //Draw one dot there.
      imagesetpixel($newImage,$x,$y,$newColor) or stegmuffin_imgerr('add new color at ('.$x.','.$y.')');
      imagecolordeallocate($newImage,$newColor);
    }
  }
  imagedestroy($hidImage);
  imagedestroy($covImage);
  //It's a PNG.
  header('Content-type: image/png');
  //If you want to download it rather than view it ...
  if($dload) header("Content-Disposition: attachment; filename=stegmuffin-hide.png");
  //Display the image.
  imagepng($newImage);
  //End of line.
  die;
}
?>